@@ -30,21 +30,24 @@
   deleteService.$inject = [
     '$location',
     '$q',
+    '$rootScope',
     'horizon.app.core.openstack-service-api.zun',
     'horizon.app.core.openstack-service-api.policy',
     'horizon.framework.util.actions.action-result.service',
     'horizon.framework.util.i18n.gettext',
     'horizon.framework.util.q.extensions',
     'horizon.framework.widgets.modal.deleteModalService',
+    'horizon.framework.widgets.table.events',
     'horizon.framework.widgets.toast.service',
+    'horizon.dashboard.container.containers.adminActions',
     'horizon.dashboard.container.containers.resourceType',
     'horizon.dashboard.container.containers.events',
     'horizon.dashboard.container.containers.validStates'
   ];
 
   function deleteService(
-    $location, $q, zun, policy, actionResult, gettext, $qExtensions, deleteModal,
-    toast, resourceType, events, validStates
+    $location, $q, $rootScope, zun, policy, actionResult, gettext, $qExtensions, deleteModal,
+    tableEvents, toast, adminActions, resourceType, events, validStates
   ) {
     var scope;
     var context = {
@@ -67,15 +70,24 @@
     }
 
     function allowed(container) {
+      var adminAction = true;
+      if (zun.isAdmin()) {
+        adminAction = adminActions.indexOf("delete") >= 0;
+      }
       // only row actions pass in container
       // otherwise, assume it is a batch action
+      var state;
       if (container) {
-        return $qExtensions.booleanAsPromise(
+        state = $qExtensions.booleanAsPromise(
           validStates.delete.indexOf(container.status) >= 0
         );
       } else {
-        return $qExtensions.booleanAsPromise(true);
+        state = $qExtensions.booleanAsPromise(true);
       }
+      return $q.all([
+        $qExtensions.booleanAsPromise(adminAction),
+        state
+      ]);
     }
 
     // delete selected resource objects
@@ -110,10 +122,10 @@
 
     // for batch delete
     function afterCheck(result) {
-      var outcome = $q.reject();  // Reject the promise by default
+      var outcome = $q.reject().catch(angular.noop);  // Reject the promise by default
       if (result.fail.length > 0) {
         toast.add('error', getMessage(notAllowedMessage, result.fail));
-        outcome = $q.reject(result.fail);
+        outcome = $q.reject(result.fail).catch(angular.noop);
       }
       if (result.pass.length > 0) {
         outcome = deleteModal.open(scope, result.pass.map(getEntity), context).then(createResult);
@@ -126,17 +138,18 @@
       // from the deleteModal into a standard form
       var result = actionResult.getActionResult();
       deleteModalResult.pass.forEach(function markDeleted(item) {
-        result.deleted(resourceType, getEntity(item).id);
+        result.updated(resourceType, getEntity(item).id);
       });
       deleteModalResult.fail.forEach(function markFailed(item) {
         result.failed(resourceType, getEntity(item).id);
       });
       var indexPath = '/project/container/containers';
       var currentPath = $location.path();
-      if (result.result.failed.length === 0 && result.result.deleted.length > 0 &&
+      if (result.result.failed.length === 0 && result.result.updated.length > 0 &&
           currentPath !== indexPath) {
         $location.path(indexPath);
       } else {
+        $rootScope.$broadcast(tableEvents.CLEAR_SELECTIONS);
         return result.result;
       }
     }
