@@ -22,17 +22,23 @@
   workflow.$inject = [
     "horizon.app.core.openstack-service-api.cinder",
     "horizon.app.core.openstack-service-api.neutron",
+    "horizon.app.core.openstack-service-api.security-group",
+    'horizon.app.core.openstack-service-api.zun',
     "horizon.dashboard.container.basePath",
     "horizon.framework.util.i18n.gettext",
+    'horizon.framework.util.q.extensions',
     "horizon.framework.widgets.metadata.tree.service"
   ];
 
-  function workflow(cinder, neutron, basePath, gettext, treeService) {
+  function workflow(
+    cinder, neutron, securityGroup, zun, basePath, gettext,
+    $qExtensions, treeService
+  ) {
     var workflow = {
       init: init
     };
 
-    function init(action, title, submitText) {
+    function init(action, title, submitText, id) {
       var push = Array.prototype.push;
       var schema, form, model;
       var imageDrivers = [
@@ -45,16 +51,16 @@
         {value: "always", name: gettext("Always")},
         {value: "never", name: gettext("Never")}
       ];
-      var restartPolicies = [
+      var exitPolicies = [
         {value: "", name: gettext("Select policy.")},
         {value: "no", name: gettext("No")},
-        {value: "on-failure", name: gettext("On failure")},
-        {value: "always", name: gettext("Always")},
-        {value: "unless-stopped", name: gettext("Unless Stopped")}
+        {value: "on-failure", name: gettext("Restart on failure")},
+        {value: "always", name: gettext("Always restart")},
+        {value: "unless-stopped", name: gettext("Restart if stopped")},
+        {value: "remove", name: gettext("Remove container")}
       ];
-      var runtimes = [
-        {value: "", name: gettext("Select runtime.")},
-        {value: "runc", name: gettext("runc")}
+      var availabilityZones = [
+        {value: "", name: gettext("Select availability zone.")}
       ];
 
       // schema
@@ -91,6 +97,10 @@
             title: gettext("Hostname"),
             type: "string"
           },
+          runtime: {
+            title: gettext("Runtime"),
+            type: "string"
+          },
           cpu: {
             title: gettext("CPU"),
             type: "number",
@@ -101,12 +111,17 @@
             type: "number",
             minimum: 4
           },
-          destination: {
-            title: gettext("Destination"),
+          disk: {
+            title: gettext("Disk"),
+            type: "number",
+            minimum: 0
+          },
+          availability_zone: {
+            title: gettext("Availability Zone"),
             type: "string"
           },
-          restart_policy: {
-            title: gettext("Restart Policy"),
+          exit_policy: {
+            title: gettext("Exit Policy"),
             type: "string"
           },
           restart_policy_max_retry: {
@@ -114,9 +129,9 @@
             type: "number",
             minimum: 0
           },
-          runtime: {
-            title: gettext("Runtime"),
-            type: "string"
+          auto_heal: {
+            title: gettext("Enable auto heal"),
+            type: "boolean"
           },
           // misc
           workdir: {
@@ -131,10 +146,6 @@
             title: gettext("Enable interactive mode"),
             type: "boolean"
           },
-          auto_remove: {
-            title: gettext("Auto remove"),
-            type: "boolean"
-          },
           // labels
           labels: {
             title: gettext("Labels"),
@@ -218,7 +229,7 @@
               items: [
                 {
                   type: "section",
-                  htmlClass: "col-xs-12",
+                  htmlClass: "col-xs-6",
                   items: [
                     {
                       key: "hostname",
@@ -227,6 +238,17 @@
                     }
                   ]
                 },
+                {
+                  type: "section",
+                  htmlClass: "col-xs-6",
+                  items: [
+                    {
+                      key: "runtime",
+                      placeholder: gettext("The runtime to create container with."),
+                      readonly: action === "update"
+                    }
+                  ]
+                },
                 {
                   type: "section",
                   htmlClass: "col-xs-6",
@@ -244,6 +266,7 @@
                   items: [
                     {
                       key: "memory",
+                      step: 128,
                       placeholder: gettext("The container memory size in MiB.")
                     }
                   ]
@@ -253,16 +276,48 @@
                   htmlClass: "col-xs-6",
                   items: [
                     {
-                      key: "restart_policy",
+                      key: "disk",
+                      step: 1,
+                      placeholder: gettext("The disk size in GiB for per container.")
+                    }
+                  ]
+                },
+                {
+                  type: "section",
+                  htmlClass: "col-xs-6",
+                  items: [
+                    {
+                      key: "availability_zone",
+                      readonly: action === "update",
+                      type: "select",
+                      titleMap: availabilityZones
+                    }
+                  ]
+                },
+                {
+                  type: "section",
+                  htmlClass: "col-xs-6",
+                  items: [
+                    {
+                      key: "exit_policy",
                       type: "select",
                       readonly: action === "update",
-                      titleMap: restartPolicies,
+                      titleMap: exitPolicies,
                       onChange: function() {
-                        var readonly = model.restart_policy !== "on-failure";
-                        if (readonly) {
+                        var notOnFailure = model.exit_policy !== "on-failure";
+                        if (notOnFailure) {
                           model.restart_policy_max_retry = "";
                         }
-                        form[0].tabs[1].items[4].items[0].readonly = readonly;
+                        form[0].tabs[1].items[7].items[0].readonly = notOnFailure;
+                        // set auto_remove whether exit_policy is "remove".
+                        // if exit_policy is set as "remove", clear restart_policy.
+                        // otherwise, set restart_policy as same value as exit_policy.
+                        model.auto_remove = (model.exit_policy === "remove");
+                        if (model.auto_remove) {
+                          model.restart_policy = "";
+                        } else {
+                          model.restart_policy = model.exit_policy;
+                        }
                       }
                     }
                   ]
@@ -273,20 +328,18 @@
                   items: [
                     {
                       key: "restart_policy_max_retry",
-                      placeholder: gettext("Retry times for 'On failure' policy."),
+                      placeholder: gettext("Retry times for 'Restart on failure' policy."),
                       readonly: true
                     }
                   ]
                 },
                 {
                   type: "section",
-                  htmlClass: "col-xs-6",
+                  htmlClass: "col-xs-12",
                   items: [
                     {
-                      key: "runtime",
-                      type: "select",
-                      readonly: action === "update",
-                      titleMap: runtimes
+                      key: "auto_heal",
+                      readonly: action === "update"
                     }
                   ]
                 }
@@ -327,8 +380,7 @@
                     }
                   ]
                 }
-              ],
-              condition: action === "update"
+              ]
             },
             {
               "title": gettext("Ports"),
@@ -395,10 +447,6 @@
                     {
                       key: "interactive",
                       readonly: action === "update"
-                    },
-                    {
-                      key: "auto_remove",
-                      readonly: action === "update"
                     }
                   ]
                 }
@@ -460,11 +508,16 @@
         run: true,
         // spec
         hostname: "",
+        runtime: "",
         cpu: "",
         memory: "",
+        disks: "",
+        availability_zone: "",
+        exit_policy: "",
         restart_policy: "",
         restart_policy_max_retry: "",
-        runtime: "",
+        auto_remove: false,
+        auto_heal: false,
         // mounts
         mounts: [],
         // networks
@@ -477,7 +530,6 @@
         workdir: "",
         environment: "",
         interactive: true,
-        auto_remove: false,
         // labels
         labels: "",
         // hints
@@ -494,14 +546,101 @@
         {id: "", name: gettext("Select available Cinder volume")}
       ];
 
-      // available networks
+      // networks
       model.availableNetworks = [];
+      model.allocatedNetworks = [];
 
       // available ports
       model.availablePorts = [];
 
+      // security groups
+      model.availableSecurityGroups = [];
+      model.allocatedSecurityGroups = [];
+
+      // get resources
+      getContainer(action, id).then(function () {
+        getVolumes();
+        getNetworks();
+        securityGroup.query().then(onGetSecurityGroups);
+        zun.getZunAvailabilityZones().then(onGetZunServices);
+      });
+
+      // get container when action equals "update"
+      function getContainer (action, id) {
+        if (action === 'create') {
+          return $qExtensions.booleanAsPromise(true);
+        } else {
+          return zun.getContainer(id).then(onGetContainer);
+        }
+      }
+
+      // get container for update
+      function onGetContainer(response) {
+        model.id = id;
+        model.name = response.data.name
+          ? response.data.name : "";
+        model.image = response.data.image
+          ? response.data.image : "";
+        model.image_driver = response.data.image_driver
+          ? response.data.image_driver : "docker";
+        model.image_pull_policy = response.data.image_pull_policy
+          ? response.data.image_pull_policy : "";
+        model.command = response.data.command
+          ? response.data.command : "";
+        model.hostname = response.data.hostname
+          ? response.data.hostname : "";
+        model.runtime = response.data.runtime
+          ? response.data.runtime : "";
+        model.cpu = response.data.cpu
+          ? response.data.cpu : "";
+        model.memory = response.data.memory
+          ? parseInt(response.data.memory, 10) : "";
+        model.restart_policy = response.data.restart_policy.Name
+          ? response.data.restart_policy.Name : "";
+        model.restart_policy_max_retry = response.data.restart_policy.MaximumRetryCount
+          ? parseInt(response.data.restart_policy.MaximumRetryCount, 10) : null;
+        if (config.model.auto_remove) {
+          config.model.exit_policy = "remove";
+        } else {
+          config.model.exit_policy = config.model.restart_policy;
+        }
+        model.allocatedNetworks = getAllocatedNetworks(response.data.addresses);
+        model.allocatedSecurityGroups = response.data.security_groups;
+        model.workdir = response.data.workdir
+          ? response.data.workdir : "";
+        model.environment = response.data.environment
+          ? hashToString(response.data.environment) : "";
+        model.interactive = response.data.interactive
+          ? response.data.interactive : false;
+        model.auto_remove = response.data.auto_remove
+          ? response.data.auto_remove : false;
+        model.labels = response.data.labels
+          ? hashToString(response.data.labels) : "";
+        return response;
+      }
+
+      function getAllocatedNetworks(addresses) {
+        var allocated = [];
+        Object.keys(addresses).forEach(function (id) {
+          allocated.push(id);
+        });
+        return allocated;
+      }
+
+      function hashToString(hash) {
+        var str = "";
+        for (var key in hash) {
+          if (hash.hasOwnProperty(key)) {
+            if (str.length > 0) {
+              str += ",";
+            }
+            str += key + "=" + hash[key];
+          }
+        }
+        return str;
+      }
+
       // get available cinder volumes
-      getVolumes();
       function getVolumes() {
         return cinder.getVolumes().then(onGetVolumes);
       }
@@ -519,7 +658,6 @@
       }
 
       // get available neutron networks and ports
-      getNetworks();
       function getNetworks() {
         return neutron.getNetworks().then(onGetNetworks).then(getPorts);
       }
@@ -529,6 +667,14 @@
           response.data.items.filter(function(network) {
             return network.subnets.length > 0;
           }));
+        // if network in model.allocatedNetworks, push it to mode.network for update
+        model.availableNetworks.forEach(function (available) {
+          model.allocatedNetworks.forEach(function (allocated) {
+            if (available.id === allocated) {
+              model.networks.push(available);
+            }
+          });
+        });
         return response;
       }
 
@@ -540,6 +686,7 @@
             }
           );
         });
+        return networks;
       }
 
       function onGetPorts(ports, network) {
@@ -566,6 +713,36 @@
         return subnetNames;
       }
 
+      // get security groups
+      function onGetSecurityGroups(response) {
+        angular.forEach(response.data.items, function (item) {
+          // 'default' is a special security group in neutron. It can not be
+          // deleted and is guaranteed to exist. It by default contains all
+          // of the rules needed for an instance to reach out to the network
+          // so the instance can provision itself.
+          if (item.name === 'default' && action === "create") {
+            model.security_groups.push(item);
+          }
+          // if network in model.allocatedSecurityGroups,
+          // push it to mode.security_groups for update
+          else if (model.allocatedSecurityGroups.includes(item.id)) {
+            model.security_groups.push(item);
+          }
+
+        });
+        push.apply(model.availableSecurityGroups, response.data.items);
+        return response;
+      }
+
+      // get availability zones from zun services
+      function onGetZunServices(response) {
+        var azs = [];
+        response.data.items.forEach(function (service) {
+          azs.push({value: service.availability_zone, name: service.availability_zone});
+        });
+        push.apply(availabilityZones, azs);
+      }
+
       var config = {
         title: title,
         submitText: submitText,
